@@ -17,6 +17,9 @@
 #
 # Author: Adam Williamson <awilliam@redhat.com>
 
+# Making this explicit is often much easier to read. Stupid warning.
+# pylint: disable=no-else-return
+
 """Miscellaneous small functions that don't need to be class methods."""
 
 from __future__ import unicode_literals
@@ -32,6 +35,7 @@ import os
 import re
 import shutil
 import tempfile
+# pylint:disable=import-error
 from six.moves.urllib.parse import urlparse, urlencode
 from six.moves.urllib.request import urlopen, Request
 from six.moves.urllib.error import URLError, HTTPError
@@ -40,6 +44,12 @@ import productmd.composeinfo
 
 import fedfind.const
 
+# there is a straight-up typo in productmd 1.20, fix it
+if 'vabrant-libvirt.box' in SUPPORTED_IMAGE_FORMATS:
+    SUPPORTED_IMAGE_FORMATS.remove('vabrant-libvirt.box')
+    SUPPORTED_IMAGE_FORMATS.append('vagrant-libvirt.box')
+
+# pylint:disable=invalid-name
 logger = logging.getLogger(__name__)
 
 # Cache handling stuff
@@ -48,7 +58,7 @@ try:
     if not os.path.exists(CACHEDIR):
         os.makedirs(CACHEDIR)
     if not os.access(CACHEDIR, os.W_OK):
-        raise OSError("Write access to {0} denied!".format(CACHEDIR))
+        raise OSError("Write access to {} denied!".format(CACHEDIR))
 except (IOError, OSError) as err:
     logger.warning("Cannot use permanent cache! Using temporary cache, valid only for process")
     logger.warning("Error: %s", err)
@@ -61,7 +71,7 @@ except (IOError, OSError) as err:
 
 # cache for pkgdb collections JSON, as there are cases where we want
 # to re-use it
-COLLECTIONS = None
+RELEASES = None
 
 def date_check(value, fail_raise=True, out='obj'):
     """Checks whether a value is a date, and returns it if so. Valid
@@ -80,7 +90,7 @@ def date_check(value, fail_raise=True, out='obj'):
             dateobj = datetime.datetime.strptime(str(value), '%Y%m%d')
         except ValueError:
             if fail_raise:
-                newerr = "{0} is not a valid date.".format(value)
+                newerr = "{} is not a valid date.".format(value)
                 raise ValueError(newerr)
             else:
                 return False
@@ -108,7 +118,7 @@ def urlopen_retries(url, tries=5):
             logger.debug("Error: %s", err)
             tries -= 1
     if not resp:
-        raise ValueError("urlopen_retries: Failed to open {0}!".format(url))
+        raise ValueError("urlopen_retries: Failed to open {}!".format(url))
     return resp
 
 def download_json(url):
@@ -132,7 +142,7 @@ def url_exists(url):
         except (ValueError, URLError, HTTPError):
             return False
     else:
-        raise ValueError("Invalid or unhandled URL! URL: {0}".format(url))
+        raise ValueError("Invalid or unhandled URL! URL: {}".format(url))
 
 def get_size(url):
     """Get the size of URL (Content-Length header for HTTP). Currently
@@ -154,42 +164,81 @@ def comma_list(string):
     """
     return string.lower().split(',')
 
-def _get_collections():
-    """Download the collections JSON if we haven't already, and cache
-    it at the module level.
+def _get_releases():
+    """Download the Bodhi 'releases' JSON if we haven't already, and
+    cache it at the module level. Expires after one day (so we don't
+    have to restart consumers etc. when we cut a release or branch).
     """
-    global COLLECTIONS
-    if not COLLECTIONS:
-        logger.debug("collections not cached, downloading...")
-        gotjson = download_json(
-            'https://admin.fedoraproject.org/pkgdb/api/collections/')
-        COLLECTIONS = gotjson['collections']
-    return COLLECTIONS
+    # yes, pylint, we really want a global. sheesh.
+    # pylint:disable=global-statement
+    global RELEASES
+
+    if RELEASES:
+        # check if we should expire the cache. first item in RELEASES
+        # is the datetime it was last updated
+        delta = datetime.datetime.utcnow() - RELEASES[0]
+        # this means "if it was more than a day ago"
+        if delta > datetime.timedelta(1):
+            RELEASES = []
+
+    if not RELEASES:
+        logger.debug("releases not cached or cache expired, downloading...")
+        # wonder if I'll be gone before we start paginating here!
+        gotjson = download_json('https://bodhi.fedoraproject.org/releases/?rows_per_page=500')
+        RELEASES = [datetime.datetime.utcnow(), gotjson['releases']]
+
+    # only return the actual data, not the timestamp...
+    return RELEASES[1]
 
 def get_current_release(branched=False):
     """Find out what the 'current' Fedora release is, according to
-    the pkgdb API. If 'branched' is truth-y, will return the Branched
-    release number if there is one.
+    the Bodhi 'releases' API. If 'branched' is truth-y, will return
+    the Branched release number if there is one.
     """
-    collections = _get_collections()
+    releases = _get_releases()
+    # find current releases
+    rels = {int(rel['version']) for rel in releases if
+            rel['state'] == 'current' and rel['id_prefix'] == 'FEDORA'}
     if branched:
-        rels = [coll['version'] for coll in collections if
-                coll['status'] in ('Active', 'Under Development')]
-    else:
-        rels = [coll['version'] for coll in collections if
-                coll['status'] == 'Active']
-    rels = [int(rel) for rel in rels if rel.isdigit()]
+        # find development releases. The 'isdigit' check avoids us
+        # blowing up on ELN, which has a non-digit version ("eln")
+        devrels = {int(rel['version']) for rel in releases if
+                   rel['state'] == 'pending' and rel['id_prefix'] == 'FEDORA'
+                   and rel["version"].isdigit()}
+        # if there is only one development release, it's rawhide, and
+        # we should ignore it. if there's more than one, we'll assume
+        # the *lowest numbered* is Branched
+        if len(devrels) > 1:
+            rels.add(min(devrels))
     return max(rels)
 
 def get_current_stables():
     """Returns a list of the release numbers of all current stable
-    releases (according to pkgdb).
+    releases (according to Bodhi).
     """
-    collections = _get_collections()
-    return [int(coll['version']) for coll in collections if
-            coll['status'] == 'Active' and coll['name'] == 'Fedora']
+    releases = _get_releases()
+    return [int(rel['version']) for rel in releases if
+            rel['state'] == 'current' and rel['id_prefix'] == 'FEDORA']
+
+def _type_from_supported(types, filename):
+    """We run through this twice, with slightly different candidate
+    types, because life is awful.
+    """
+    for typ in types:
+        # Some types are basically formats...
+        if '-' in typ and typ.replace('-', '.') in filename:
+            return typ
+        # Others are more token-y.
+        elif "-{}-".format(typ) in filename:
+            return typ
+    # Sometimes we have to be a bit more relaxed.
+    for typ in types:
+        if typ != "cd" and typ in filename:
+             return typ
+    return ''
 
 def create_image_dict(path):
+    # pylint:disable=too-many-branches
     """Synthesize a productmd-style image metadata dict by analyzing
     the image's path. Used for pre-Pungi 4 releases, and Pungi 4
     releases that were split and had metadata stripped when synced to
@@ -214,24 +263,22 @@ def create_image_dict(path):
 
     # Find type
     imgdict['type'] = ""
-    # we invent a types for the 'multiple' images that contain
+    # we invent types for the 'multiple' images that contain
     # multiple live or installer images together with a boot menu.
     # these will have 'Multi' subvariant
     types = SUPPORTED_IMAGE_TYPES + ['multi-desktop', 'multi-install']
-    for typ in types:
-        # Some types are basically formats...
-        if '-' in typ and typ.replace('-', '.') in filename:
-            imgdict['type'] = typ
-        # Others are more token-y.
-        elif "-{0}-".format(typ) in filename:
-            imgdict['type'] = typ
-    # Sometimes we have to be a bit more relaxed.
+    # we don't want to use the 'tar-gz' type until there's no alternative
+    # this is so we don't identify absolutely any .tar.gz image as type
+    # 'tar-gz' when another type would be more accurate
+    # ditto 'qcow2' vs 'qcow' - we want to prefer qcow2
+    types = [typ for typ in types if typ not in ('tar-gz', 'qcow')]
+    imgdict['type'] = _type_from_supported(types, filename)
     if not imgdict['type']:
-        for typ in types:
-            if typ != "cd" and typ in filename:
-                imgdict['type'] = typ
-    # and this is just magic.
+        # fine, we'll try with 'tar-gz' and 'qcow', jeez
+        types.extend(['tar-gz', 'qcow'])
+        imgdict['type'] = _type_from_supported(types, filename)
     if not imgdict['type'] and "disc" in filename:
+        # this is just magic.
         imgdict['type'] = "cd"
     # Pungi config uses the type 'boot' for netinst images, so follow
     # that
@@ -264,25 +311,22 @@ def create_image_dict(path):
     imgdict['subvariant'] = ''
     # these are a couple of nasty special cases taken
     # from fedfind 1.x
-    if re.match(r'.*Fedora-(x86_64|i386)-\d{2,2}-\d{8,8}'
-                r'(\.\d|)-sda\.(qcow2|raw.xz)',
+    if re.match(r'.*Fedora-(x86_64|i386)-\d{2,2}-\d{8,8}(\.\d|)-sda\.(qcow2|raw.xz)',
                 imgdict['path']):
         # F19/20-era cloud image, before Cloud flavor.
         imgdict['subvariant'] = 'Cloud'
 
-    if re.match(r'.*(Fedora-|F)\d{1,2}-(x86_64|i686|Live)'
-                r'-(x86_64|i686|Live)\.iso', imgdict['path']):
+    if re.match(r'.*(Fedora-|F)\d{1,2}-(x86_64|i686|Live)-(x86_64|i686|Live)\.iso',
+                imgdict['path']):
         # < F14 Desktop lives. Ew.
         imgdict['subvariant'] = 'Desktop'
 
     for load in fedfind.const.SUBVARIANTS:
-        if any(val in imgdict['path'].lower()
-               for val in load.values):
+        if any(val in imgdict['path'].lower() for val in load.values):
             imgdict['subvariant'] = load.name
     # Try to identify 'generic' media (call them
     # 'Everything' as that's what productmd does)
-    if not imgdict['subvariant'] and (imgdict['type'] in
-                                      ('dvd', 'boot', 'cd')):
+    if not imgdict['subvariant'] and (imgdict['type'] in ('dvd', 'boot', 'cd')):
         imgdict['subvariant'] = 'Everything'
 
     # assign disc number
@@ -295,6 +339,7 @@ def create_image_dict(path):
     return imgdict
 
 def pdc_query(path, params=None):
+    # pylint:disable=too-many-locals,too-many-statements
     """Light 'API client', sorta, for PDC. Just handles sending a
     request to a given API path with the appropriate header to get
     a JSON response, parsing the response, and handling multi-page
@@ -413,12 +458,18 @@ def find_cid(string):
 def parse_cid(origcid, dist=False, dic=False):
     """If dist is false-y, get (release, date, type, respin) values
     from a Pungi 4 compose id. If dist is truth-y, get (dist, release,
-    date, type, respin) values from a Pungi 4 compose id.
+    date, type, respin) values from a Pungi 4 compose id. If dic is
+    truth-y, return a dictionary containing 'short' (dist), 'version'
+    (release), 'version_type', 'base_short', 'base_version',
+    'base_type', 'variant', 'date', 'compose_type' (type) and 'respin'
+    values, some of which may be '' if not present in the compose ID.
     """
+    # again, this is just a complex operation.
+    # pylint: disable=too-many-locals,too-many-branches,too-many-statements
     # Normalize and check cid
     cid = find_cid(origcid)
     if not cid:
-        raise ValueError("{0} does not appear to be a valid Pungi 4 Fedora "
+        raise ValueError("{} does not appear to be a valid Pungi 4 Fedora "
                          "compose ID!".format(origcid))
     # Writing a truly generic compose ID parser is pretty tricky. See
     # https://github.com/release-engineering/productmd/issues/77
@@ -504,7 +555,7 @@ def parse_cid(origcid, dist=False, dic=False):
                 if elems[idx+1:idx+1+len(rtelems)] == rtelems:
                     maintype = _reltype
                     # pop out the right number of elems
-                    for i in range(0,len(rtelems)):
+                    for _ in range(0, len(rtelems)):
                         elems.pop(idx+1)
                     break
             basever = somever
@@ -554,7 +605,7 @@ def cid_from_label(release, label, short='fedora'):
     specify. Uses PDC.
     """
     short = short.lower()
-    params = {'release': "{0}-{1}".format(short, release), 'compose_label': label}
+    params = {'release': "{}-{}".format(short, release), 'compose_label': label}
     res = pdc_query('composes', params)
     if res:
         return res[0]['compose_id']
@@ -588,7 +639,8 @@ def get_weight(imgdict, arch=True):
         (('everything',), 300),
         (('workstation',), 220),
         (('server',), 210),
-        (('cloud', 'desktop', 'cloud_base', 'docker_base', 'atomic'), 200),
+        (('cloud', 'desktop', 'cloud_base', 'docker_base', 'container_base',
+          'container_minimal_base', 'atomic'), 200),
         (('kde',), 190),
         (('minimal',), 90),
         (('xfce',), 80),
@@ -686,13 +738,15 @@ def _get_filelist(flist='fedora', record=None):
     the cached file, then always iterate it out before returning the
     file handle.
     """
+    # again, this is just a complex operation.
+    # pylint: disable=too-many-branches,too-many-statements
     if record is not None:
         record['cache'] = False
         record['downfail'] = False
     # Determine the URL
     if flist not in ('fedora', 'alt', 'archive'):
         raise ValueError("_get_filelist: must be fedora, archive or alt!")
-    fname = 'imagelist-{0}'.format(flist)
+    fname = 'imagelist-{}'.format(flist)
     url = '/'.join((fedfind.const.HTTPS_DL, flist, fname))
     logger.debug("_get_filelist: url is %s", url)
 
@@ -712,8 +766,8 @@ def _get_filelist(flist='fedora', record=None):
         modified = resp.headers['last-modified']
     except ValueError:
         if cachefh:
-            logger.warning("Could not access image list {0}! Using cached "
-                           "copy. Cache may be outdated!")
+            logger.warning("Could not access image list %s at %s! Using cached "
+                           "copy. Cache may be outdated!", flist, url)
             if record:
                 record['downfail'] = True
                 record['cache'] = True
