@@ -2,10 +2,10 @@
 
 baddeps=""
 # check deps
-rpm -qi python2-setuptools > /dev/null 2>&1 || baddeps="python2-setuptools"
-rpm -qi python2-setuptools_git > /dev/null 2>&1 || baddeps="${baddeps} python2-setuptools_git"
-rpm -qi python2-pypandoc > /dev/null 2>&1 || baddeps="${baddeps} python2-pypandoc"
-rpm -qi python2-twine > /dev/null 2>&1 || baddeps="${baddeps} python2-twine"
+rpm -qi python3-setuptools > /dev/null 2>&1 || baddeps="python3-setuptools"
+rpm -qi python3-setuptools_git > /dev/null 2>&1 || baddeps="${baddeps} python3-setuptools_git"
+rpm -qi python3-pypandoc > /dev/null 2>&1 || baddeps="${baddeps} python3-pypandoc"
+rpm -qi twine > /dev/null 2>&1 || baddeps="${baddeps} twine"
 if [ -n "${baddeps}" ]; then
     echo "${baddeps} must be installed!"
     exit 1
@@ -25,6 +25,6 @@ git commit -s -m "Release ${version}"
 git push
 git tag -a -m "Release ${version}" ${version}
 git push origin ${version}
-python ./setup.py sdist --formats=tar
+python3 ./setup.py sdist --formats=tar
 gzip dist/${name}-${version}.tar
 twine upload dist/${name}-${version}.tar.gz
