@@ -19,6 +19,7 @@
 
 # these are all kinda inappropriate for pytest patterns
 # pylint: disable=old-style-class, no-init, protected-access, no-self-use, unused-argument
+# pylint: disable=too-many-public-methods
 
 """Tests for helpers.py."""
 
@@ -33,6 +34,7 @@ import shutil
 
 import mock
 import pytest
+from freezegun import freeze_time
 # pylint: disable=import-error
 from six.moves.urllib.request import Request
 
@@ -204,28 +206,41 @@ class TestHelpers:
         """It splits lists on commas. It ain't rocket science."""
         assert fedfind.helpers.comma_list('foo,Bar,moo') == ['foo', 'bar', 'moo']
 
-    def test_get_collections(self, json_collections):
-        """Test _get_collections works correctly and caches."""
-        (mocked, fakedata) = json_collections
+    def test_get_releases(self, json_releases):
+        """Test _get_releases works correctly and caches."""
+        (mocked, fakedata) = json_releases
         # download twice to check caching is working
-        colls = fedfind.helpers._get_collections()
-        colls = fedfind.helpers._get_collections()
-        assert colls == fakedata
-        assert mocked.call_count == 1
+        with freeze_time("2019-10-01"):
+            colls = fedfind.helpers._get_releases()
+            colls = fedfind.helpers._get_releases()
+            assert colls == fakedata
+            assert mocked.call_count == 1
+        # now test the cache expiry works
+        with freeze_time("2019-10-03"):
+            colls = fedfind.helpers._get_releases()
+            assert colls == fakedata
+            assert mocked.call_count == 2
 
-    def test_get_current_release(self, collections):
+    def test_get_current_release_branched(self, releases_branched):
         """Test get_current_release gives expected values, with our
-        fake collections data.
+        fake collections data (with a Branched release).
         """
-        assert fedfind.helpers.get_current_release() == 23
-        assert fedfind.helpers.get_current_release(branched=True) == 24
+        assert fedfind.helpers.get_current_release() == 31
+        assert fedfind.helpers.get_current_release(branched=True) == 32
 
-    def test_get_current_stables(self, collections):
+    def test_get_current_release_nobranched(self, releases_nobranched):
+        """Test get_current_release gives expected values, with our
+        fake collections data (with no Branched release).
+        """
+        assert fedfind.helpers.get_current_release() == 31
+        assert fedfind.helpers.get_current_release(branched=True) == 31
+
+    def test_get_current_stables(self, releases_branched):
         """Test get_current_stables gives the expected values."""
-        assert fedfind.helpers.get_current_stables() == [22, 23]
+        assert fedfind.helpers.get_current_stables() == [30, 31]
 
     @pytest.mark.parametrize(
-        ("path", "arch", "form" ,"typ", "subvariant", "disc_number"),
+        ("path", "arch", "form", "typ", "subvariant", "disc_number"),
         # the image path to synthesize a dict for, and the expected values
         [
             # old-style two-week atomic compose
@@ -263,6 +278,7 @@ class TestHelpers:
         """Check create_image_dict produces correct dicts with various
         standard and tricky cases.
         """
+        # pylint: disable=too-many-arguments
         ret = fedfind.helpers.create_image_dict(path)
         assert ret == {
             'path': path,
@@ -377,6 +393,7 @@ class TestHelpers:
         """Check parse_cid works correctly, with various known compose
         ID forms.
         """
+        # pylint: disable=too-many-arguments
         assert fedfind.helpers.parse_cid(cid, dic=True) == {
             'short': short,
             'version': version,
@@ -404,7 +421,7 @@ class TestHelpers:
     def test_parse_cid_error(self):
         """Check parse_cid raises ValueError for non-Pungi 4-ish CID."""
         with pytest.raises(ValueError):
-            res = fedfind.helpers.parse_cid('23-20160530')
+            fedfind.helpers.parse_cid('23-20160530')
 
     @mock.patch('fedfind.helpers.download_json', return_value=PDC_JSON_REAL, autospec=True)
     def test_cid_from_label(self, fakejson):
@@ -456,7 +473,7 @@ class TestHelpers:
         def check_file(flist, touch=False):
             """Inner function for doing the check (run multiple times)"""
             origfile = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'http',
-                                    'pub', flist, 'imagelist-{0}'.format(flist))
+                                    'pub', flist, 'imagelist-{}'.format(flist))
             if touch:
                 os.utime(origfile, None)
             gotcksum = hashlib.md5()
@@ -464,7 +481,7 @@ class TestHelpers:
             record = {}
             with fedfind.helpers._get_filelist(flist=flist, record=record) as testfh:
                 assert testfh.name == os.path.join(clean_home, '.cache', 'fedfind',
-                                                   'imagelist-{0}'.format(flist))
+                                                   'imagelist-{}'.format(flist))
                 for line in testfh:
                     gotcksum.update(line.encode('utf-8'))
             with codecs.open(origfile, encoding='utf-8') as origfh:
