@@ -19,6 +19,7 @@
 
 # these are all kinda inappropriate for pytest patterns
 # pylint: disable=old-style-class, no-init, protected-access, no-self-use, unused-argument
+# pylint: disable=invalid-name, too-few-public-methods, too-many-public-methods, too-many-lines
 
 """Tests for release.py."""
 
@@ -32,6 +33,8 @@ import os
 import pytest
 
 import mock
+import six
+
 import fedfind.release
 
 # Some chunks of HTML from mirror package directories for testing
@@ -214,8 +217,9 @@ class FakeResponsePackages(object):
         """Returns the appropriate chunk of HTML (see above) if the
         correct URL was given.
         """
-        arcbase = 'https://download.fedoraproject.org/pub/archive/fedora/linux/'
+        arcbase = '{}/archive/fedora/linux/'.format(fedfind.const.HTTPS)
         branchbase = 'https://kojipkgs.fedoraproject.org/compose/branched/Fedora-24-20160321.n.0/'
+        modbase = 'https://kojipkgs.fedoraproject.org/compose/Fedora-Modular-27-20171115.n.1/'
         mapping = {
             # we just pretend 1 is 6, it's easier
             arcbase + 'core/1/SRPMS/': FC6PACKAGEHTML,
@@ -224,14 +228,18 @@ class FakeResponsePackages(object):
             arcbase + 'releases/18/Everything/source/SRPMS/a/': F18PACKAGEHTMLA,
             arcbase + 'releases/18/Everything/source/SRPMS/b/': F18PACKAGEHTMLB,
             arcbase + 'releases/18/Everything/source/SRPMS/f/': '',
+            # similarly we just use the 18 content for 24 and mod27
             branchbase + 'compose/Everything/source/tree/Packages/a/': F18PACKAGEHTMLA,
             branchbase + 'compose/Everything/source/tree/Packages/b/': F18PACKAGEHTMLB,
             branchbase + 'compose/Everything/source/tree/Packages/f/': '',
+            modbase + 'compose/Server/source/tree/Packages/a/': F18PACKAGEHTMLA,
+            modbase + 'compose/Server/source/tree/Packages/b/': F18PACKAGEHTMLB,
+            modbase + 'compose/Server/source/tree/Packages/f/': '',
         }
         if self.url in mapping:
-            return mapping[self.url]
+            return six.b(mapping[self.url])
         else:
-            raise ValueError("URL {0} not expected!".format(self.url))
+            raise ValueError("URL {} not expected!".format(self.url))
 
 def urlopen_fake_package(url):
     """Stub for urlopen_retries which returns FakeResponsePackages."""
@@ -261,15 +269,36 @@ class TestRelease:
             # Rawhide nightly.
             ('Rawhide', '', 'datestr', 2, "Fedora",
              fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '2'),
-            # Atomic nightly.
+            # Atomic nightly (old, milestone way).
             (24, 'Atomic', 'datestr', 0, "Fedora",
-             fedfind.release.AtomicNightly, '24', 'Atomic', 'datestr', '0'),
-            # Docker nightly.
+             fedfind.release.AtomicNightly, '24', 'Nightly', 'datestr', '0'),
+            # Docker nightly (old, milestone way).
             (25, 'Docker', 'datestr', 0, "Fedora",
-             fedfind.release.DockerNightly, '25', 'Docker', 'datestr', '0'),
-            # Cloud nightly.
+             fedfind.release.DockerNightly, '25', 'Nightly', 'datestr', '0'),
+            # Cloud nightly (old, milestone way).
             (25, 'Cloud', 'datestr', 0, "Fedora",
-             fedfind.release.CloudNightly, '25', 'Cloud', 'datestr', '0'),
+             fedfind.release.CloudNightly, '25', 'Nightly', 'datestr', '0'),
+            # Atomic nightly (new, dist way).
+            (24, '', 'datestr', 0, "Fedora-Atomic",
+             fedfind.release.AtomicNightly, '24', 'Nightly', 'datestr', '0'),
+            # Docker nightly (new, dist way).
+            (25, '', 'datestr', 0, "Fedora-Docker",
+             fedfind.release.DockerNightly, '25', 'Nightly', 'datestr', '0'),
+            # Cloud nightly (new, dist way).
+            (25, '', 'datestr', 0, "Fedora-Cloud",
+             fedfind.release.CloudNightly, '25', 'Nightly', 'datestr', '0'),
+            # Container nightly (new, dist way).
+            (28, '', 'datestr', 0, "Fedora-Container",
+             fedfind.release.ContainerNightly, '28', 'Nightly', 'datestr', '0'),
+            # IoT nightly.
+            (28, '', 'datestr', 0, "Fedora-IoT",
+             fedfind.release.IoTNightly, '28', 'Nightly', 'datestr', '0'),
+            # Updates nightly.
+            (28, 'Updates', 'datestr', 0, "Fedora",
+             fedfind.release.Updates, '28', 'Updates', 'datestr', '0'),
+            # Updates-testing nightly.
+            (28, 'Updates-testing', 'datestr', 0, "Fedora",
+             fedfind.release.UpdatesTesting, '28', 'Updates-testing', 'datestr', '0'),
             # Wikitcms format Rawhide.
             ('24', 'Rawhide', 'datestr.n.0', None, "Fedora",
              fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
@@ -279,18 +308,28 @@ class TestRelease:
              fedfind.release.BranchedNightly, '23', 'Branched', 'datestr', '0'),
             (27, 'Branched', '', 0, "Fedora-Modular",
              fedfind.release.BranchedModularNightly, '27', 'Branched', 'datestr', '0'),
-            ('Rawhide', '', '', 0, "Fedora", fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
+            ('Rawhide', '', '', 0, "Fedora",
+             fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
             ('Rawhide', '', '', 0, "Fedora-Modular",
              fedfind.release.RawhideModularNightly, 'Rawhide', '', 'datestr', '0'),
             ('Bikeshed', '', '', 0, "Fedora-Modular",
              fedfind.release.BikeshedModularNightly, 'Bikeshed', '', 'datestr', '0'),
-            (24, 'Atomic', '', 0, "Fedora", fedfind.release.AtomicNightly, '24', 'Atomic', 'datestr', '0'),
-            (25, 'Docker', '', 0, "Fedora", fedfind.release.DockerNightly, '25', 'Docker', 'datestr', '0'),
-            (25, 'Cloud', '', 0, "Fedora", fedfind.release.CloudNightly, '25', 'Cloud', 'datestr', '0'),
+            (24, 'Atomic', '', 0, "Fedora",
+             fedfind.release.AtomicNightly, '24', 'Nightly', 'datestr', '0'),
+            (25, 'Docker', '', 0, "Fedora",
+             fedfind.release.DockerNightly, '25', 'Nightly', 'datestr', '0'),
+            (25, 'Cloud', '', 0, "Fedora",
+             fedfind.release.CloudNightly, '25', 'Nightly', 'datestr', '0'),
+            (28, '', '', 0, "Fedora-Container",
+             fedfind.release.ContainerNightly, '28', 'Nightly', 'datestr', '0'),
+            (28, '', '', 0, "Fedora-IoT",
+             fedfind.release.IoTNightly, '28', 'Nightly', 'datestr', '0'),
             # Default, check None is handled.
-            (None, None, None, 0, "Fedora", fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
+            (None, None, None, 0, "Fedora",
+             fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
         ]
     )
+    # pylint:disable=too-many-arguments
     def test_get_release_simple(self, release, milestone, compose, respin, dist, expclass, exprel,
                                 expmile, expcomp, expspin):
         """Tests for get_release that require no online guessing or
@@ -350,6 +389,8 @@ class TestRelease:
     @mock.patch('fedfind.helpers.label_from_cid', return_value="Alpha-1.1", autospec=True)
     @mock.patch('fedfind.helpers.cid_from_label', autospec=True)
     @mock.patch('fedfind.release.Compose.exists', False)
+    # this is to avoid a network trip for the get_release CID check
+    @mock.patch('fedfind.release.Production.cid', 'Fedora-24-20160314.0')
     def test_get_release_cid_promote_fail(self, fakecid, fakelab):
         """This tests a rather snaky path: we call get_release with
         a CID and promote=True; the specified compose has a label, so
@@ -364,6 +405,7 @@ class TestRelease:
         assert fakecid.call_count == 0
 
     @mock.patch('fedfind.release.ModularProduction.exists', True)
+    @mock.patch('fedfind.release.ModularProduction.cid', 'Fedora-Modular-27-20171107.1')
     def test_get_release_modular_production_cid(self):
         """Test getting a ModularProduction instance via compose ID."""
         ret = fedfind.release.get_release(cid='Fedora-Modular-27-20171107.1')
@@ -375,6 +417,7 @@ class TestRelease:
         ret = fedfind.release.get_release(27, 'Beta', 1, 1, dist="Fedora-Modular")
         assert isinstance(ret, fedfind.release.ModularCompose)
 
+    @mock.patch('fedfind.release.AtomicNightly.cid', 'Fedora-Atomic-24-20160628.1')
     def test_get_release_cid_atomic_new(self):
         """Test that nightly stable Atomic compose IDs are handled.
         They have 'Fedora-Atomic' as the product name instead of
@@ -386,6 +429,7 @@ class TestRelease:
         assert ret.compose == '20160628'
         assert ret.respin == '1'
 
+    @mock.patch('fedfind.release.DockerNightly.cid', 'Fedora-Docker-25-20170113.2')
     def test_get_release_cid_docker(self):
         """Test that nightly stable Docker compose IDs are handled.
         They have 'Fedora-Docker' as the product name instead of
@@ -397,6 +441,31 @@ class TestRelease:
         assert ret.compose == '20170113'
         assert ret.respin == '2'
 
+    @mock.patch('fedfind.release.ContainerNightly.cid', 'Fedora-Container-28-20180605.0')
+    def test_get_release_cid_container(self):
+        """Test that nightly stable Container compose IDs are handled.
+        They have 'Fedora-Container' as the product name instead of
+        'Fedora'.
+        """
+        ret = fedfind.release.get_release(cid='Fedora-Container-28-20180605.0')
+        assert isinstance(ret, fedfind.release.ContainerNightly)
+        assert ret.release == '28'
+        assert ret.compose == '20180605'
+        assert ret.respin == '0'
+
+    @mock.patch('fedfind.release.IoTNightly.cid', 'Fedora-IoT-28-20180605.0')
+    def test_get_release_cid_iot(self):
+        """Test that nightly stable Docker compose IDs are handled.
+        They have 'Fedora-Docker' as the product name instead of
+        'Fedora'.
+        """
+        ret = fedfind.release.get_release(cid='Fedora-IoT-28-20180605.0')
+        assert isinstance(ret, fedfind.release.IoTNightly)
+        assert ret.release == '28'
+        assert ret.compose == '20180605'
+        assert ret.respin == '0'
+
+    @mock.patch('fedfind.release.CloudNightly.cid', 'Fedora-Cloud-26-20170217.3')
     def test_get_release_cid_cloud(self):
         """Test that nightly stable Cloud compose IDs are handled.
         They have 'Fedora-Cloud' as the product name instead of
@@ -476,6 +545,30 @@ class TestRelease:
         assert ret.compose == '20170816'
         assert ret.respin == '0'
 
+    def test_get_release_url_cid_updates(self):
+        """Getting a release based on a correct updates compose
+        URL should work.
+        """
+        url = ('https://kojipkgs.fedoraproject.org/'
+               'compose/updates/Fedora-28-updates-20180605.0/compose/')
+        ret = fedfind.release.get_release(url=url)
+        assert isinstance(ret, fedfind.release.Updates)
+        assert ret.release == '28'
+        assert ret.compose == '20180605'
+        assert ret.respin == '0'
+
+    def test_get_release_url_cid_updates_testing(self):
+        """Getting a release based on a correct updates-testing
+        compose URL should work.
+        """
+        url = ('https://kojipkgs.fedoraproject.org/'
+               'compose/updates/Fedora-28-updates-testing-20180605.0/compose/')
+        ret = fedfind.release.get_release(url=url)
+        assert isinstance(ret, fedfind.release.UpdatesTesting)
+        assert ret.release == '28'
+        assert ret.compose == '20180605'
+        assert ret.respin == '0'
+
     def test_get_release_url_cid_unknown(self):
         """Getting a release based on an unknown URL should return
         a generic Pungi4Release instance, so long as the URL starts
@@ -495,6 +588,58 @@ class TestRelease:
         with pytest.raises(ValueError):
             fedfind.release.get_release(url=url)
 
+    def test_get_release_url_check(self):
+        """Test the URL match check for get_release: if we get a
+        release by URL, but the URL of the discovered Release instance
+        doesn't match the requested URL, an exception should occur.
+        """
+        with pytest.raises(fedfind.exceptions.UrlMatchError):
+            url = 'https://kojipkgs.fedoraproject.org/wrong/Fedora-Rawhide-20180117.n.0/compose/'
+            fedfind.release.get_release(url=url)
+
+    # This would be the correct compose ID value for a Rawhide nightly
+    @mock.patch('fedfind.release.RawhideNightly.cid', 'Fedora-Rawhide-20180117.n.0')
+    def test_get_release_cid_check(self):
+        """Test the CID match check for get_release: if we get a
+        release by CID, but the CID of the discovered Release instance
+        doesn't match the requested CID, an exception should occur.
+        """
+        with pytest.raises(fedfind.exceptions.CidMatchError):
+            # real compose IDs for Rawhide are always '.n.', as we
+            # patch above; get_release will successfully handle this
+            # CID, though, and give us a RawhideNightly instance
+            cid = 'Fedora-Rawhide-20180117.0'
+            fedfind.release.get_release(cid=cid)
+        # now check it works if we do it right
+        cid = 'Fedora-Rawhide-20180117.n.0'
+        ret = fedfind.release.get_release(cid=cid)
+        assert isinstance(ret, fedfind.release.RawhideNightly)
+        assert ret.release == 'Rawhide'
+        assert ret.compose == '20180117'
+        assert ret.respin == '0'
+
+    # these mocks avoid network round trips to figure out that the
+    # compose does not exist, and to try and get the compose ID from
+    # the compose location
+    @mock.patch('fedfind.helpers.urlopen_retries', autospec=True, side_effect=ValueError)
+    @mock.patch('fedfind.release.Production.exists', False)
+    def test_get_release_cid_pdc(self, fakeurl, pdc):
+        """Test getting a release by compose ID when that release is a
+        Pungi 4 compose that has been garbage-collected, but was
+        imported to PDC. We have a special code path here which sucks
+        in the compose metadata from PDC.
+        """
+        # use a compose our pdc test fixture has data for
+        ret = fedfind.release.get_release(cid='Fedora-27-20171105.0')
+        assert isinstance(ret, fedfind.release.Production)
+        assert ret.release == '27'
+        assert ret.compose == '1'
+        assert ret.respin == '6'
+        assert ret.label == 'RC-1.6'
+        # this tests we got the metadata right - there are 84 images
+        # in the canned metadata the pdc fixture uses
+        assert len(ret.all_images) == 84
+
     @mock.patch('fedfind.release.Compose.exists', True)
     def test_get_release_compose(self):
         """A production/candidate compose. This tests the case where
@@ -521,28 +666,30 @@ class TestRelease:
         assert isinstance(ret, fedfind.release.Production)
         fakecid.assert_called_with(24, 'Alpha-1.5')
 
-    @mock.patch('fedfind.release.ArchiveRelease.exists', False)
+    @mock.patch('fedfind.release.CurrentRelease.exists', True)
     def test_get_stable_current(self):
-        """Ensure that if ArchiveRelease doesn't exist, get_release
-        for a recent stable returns CurrentRelease. Tested number must
-        be higher than the 'just return ArchiveRelease' cutoff, so
-        bump this test when bumping that cutoff.
+        """Ensure that if it exists, get_release for a recent stable
+        returns CurrentRelease. Tested number must be higher than the
+        'just return ArchiveRelease' cutoff, so bump this test when
+        bumping that cutoff.
         """
-        ret = fedfind.release.get_release(23)
+        ret = fedfind.release.get_release(31)
         assert isinstance(ret, fedfind.release.CurrentRelease)
-        assert ret.release == '23'
+        assert ret.release == '31'
 
+    @mock.patch('fedfind.release.CurrentRelease.exists', False)
     @mock.patch('fedfind.release.ArchiveRelease.exists', True)
     def test_get_stable_archive(self):
-        """Ensure that if ArchiveRelease does exist, get_release for
-        a recent stable returns it. Tested number must be higher than
-        the 'just return ArchiveRelease' cutoff (or else this test
-        will pass even if the feature is broken), so bump this test
-        when bumping that cutoff.
+        """Ensure that if CurrentRelease does not exist but
+        ArchiveRelease does exist, get_release for a recent stable
+        returns it. Tested number must be higher than the 'just return
+        ArchiveRelease' cutoff (or else this test will pass even if
+        the feature is broken), so bump this test when bumping that
+        cutoff.
         """
-        ret = fedfind.release.get_release(23)
+        ret = fedfind.release.get_release(26)
         assert isinstance(ret, fedfind.release.ArchiveRelease)
-        assert ret.release == '23'
+        assert ret.release == '26'
 
     @mock.patch('fedfind.helpers.get_current_release', return_value=23, autospec=True)
     def test_get_release_guess_release(self, fakecurrent):
@@ -559,21 +706,64 @@ class TestRelease:
         assert got.compose == datestr
         assert got.release == '24'
 
+        # old 'milestone' style
         got = fedfind.release.get_release('', 'Atomic', '', 0)
         assert isinstance(got, fedfind.release.AtomicNightly)
         assert got.compose == datestr
         assert got.release == '23'
 
+        # old 'milestone' style
         got = fedfind.release.get_release('', 'Docker', '', 0)
         assert isinstance(got, fedfind.release.DockerNightly)
         assert got.compose == datestr
         assert got.release == '23'
 
+        # old 'milestone' style
         got = fedfind.release.get_release('', 'Cloud', '', 0)
         assert isinstance(got, fedfind.release.CloudNightly)
         assert got.compose == datestr
         assert got.release == '23'
 
+        # new 'dist' style
+        got = fedfind.release.get_release('', '', '', 0, dist='Fedora-Atomic')
+        assert isinstance(got, fedfind.release.AtomicNightly)
+        assert got.compose == datestr
+        assert got.release == '23'
+
+        # new 'dist' style
+        got = fedfind.release.get_release('', '', '', 0, dist='Fedora-Docker')
+        assert isinstance(got, fedfind.release.DockerNightly)
+        assert got.compose == datestr
+        assert got.release == '23'
+
+        # new 'dist' style
+        got = fedfind.release.get_release('', '', '', 0, dist='Fedora-Cloud')
+        assert isinstance(got, fedfind.release.CloudNightly)
+        assert got.compose == datestr
+        assert got.release == '23'
+
+        got = fedfind.release.get_release('', '', '', 0, dist='Fedora-Container')
+        assert isinstance(got, fedfind.release.ContainerNightly)
+        assert got.compose == datestr
+        assert got.release == '23'
+
+        got = fedfind.release.get_release('', '', '', 0, dist='Fedora-IoT')
+        assert isinstance(got, fedfind.release.IoTNightly)
+        assert got.compose == datestr
+        assert got.release == '23'
+
+        # updates
+        got = fedfind.release.get_release('', 'Updates', '', 0)
+        assert isinstance(got, fedfind.release.Updates)
+        assert got.compose == datestr
+        assert got.release == '23'
+
+        # updates-testing
+        got = fedfind.release.get_release('', 'Updates-testing', '', 0)
+        assert isinstance(got, fedfind.release.UpdatesTesting)
+        assert got.compose == datestr
+        assert got.release == '23'
+
     @mock.patch('fedfind.release.RawhideNightly', FakeRelease)
     @mock.patch('fedfind.release.RawhideModularNightly', FakeRelease)
     @mock.patch('fedfind.release.BikeshedModularNightly', FakeRelease)
@@ -582,6 +772,10 @@ class TestRelease:
     @mock.patch('fedfind.release.AtomicNightly', FakeRelease)
     @mock.patch('fedfind.release.DockerNightly', FakeRelease)
     @mock.patch('fedfind.release.CloudNightly', FakeRelease)
+    @mock.patch('fedfind.release.ContainerNightly', FakeRelease)
+    @mock.patch('fedfind.release.IoTNightly', FakeRelease)
+    @mock.patch('fedfind.release.Updates', FakeRelease)
+    @mock.patch('fedfind.release.UpdatesTesting', FakeRelease)
     def test_get_release_guess_respin(self):
         """Test the 'respin guessing' code works (when you request a
         nightly but don't provide a respin number). Uses the fake
@@ -602,6 +796,7 @@ class TestRelease:
         got = fedfind.release.get_release('Bikeshed', '', '20171013', dist='Fedora-Modular')
         assert isinstance(got, fedfind.release.BikeshedModularNightly)
         assert got.respin == 3
+        # old 'milestone' style
         got = fedfind.release.get_release(24, 'Atomic', '20160628')
         assert isinstance(got, fedfind.release.AtomicNightly)
         assert got.respin == 3
@@ -611,6 +806,30 @@ class TestRelease:
         got = fedfind.release.get_release(24, 'Cloud', '20160628')
         assert isinstance(got, fedfind.release.CloudNightly)
         assert got.respin == 3
+        # new 'dist' style
+        got = fedfind.release.get_release(24, '', '20160628', dist='Fedora-Atomic')
+        assert isinstance(got, fedfind.release.AtomicNightly)
+        assert got.respin == 3
+        got = fedfind.release.get_release(24, '', '20160628', dist='Fedora-Docker')
+        assert isinstance(got, fedfind.release.DockerNightly)
+        assert got.respin == 3
+        got = fedfind.release.get_release(24, '', '20160628', dist='Fedora-Cloud')
+        assert isinstance(got, fedfind.release.CloudNightly)
+        assert got.respin == 3
+        got = fedfind.release.get_release(24, '', '20160628', dist='Fedora-Container')
+        assert isinstance(got, fedfind.release.ContainerNightly)
+        assert got.respin == 3
+        got = fedfind.release.get_release(24, '', '20160628', dist='Fedora-IoT')
+        assert isinstance(got, fedfind.release.IoTNightly)
+        assert got.respin == 3
+        # updates
+        got = fedfind.release.get_release(24, 'Updates', '20160628')
+        assert isinstance(got, fedfind.release.Updates)
+        assert got.respin == 3
+        # updates-testing
+        got = fedfind.release.get_release(24, 'UpdatesTesting', '20160628')
+        assert isinstance(got, fedfind.release.UpdatesTesting)
+        assert got.respin == 3
 
     @mock.patch.object(fedfind.release.BranchedNightly, 'exists', True)
     @mock.patch.object(fedfind.release.BranchedNightly, 'status', 'FINISHED')
@@ -682,22 +901,26 @@ class TestRelease:
         web server here as RespinRelease has to discover its own
         release and compose from the image list.
         """
+        # old milestone style
         got = fedfind.release.get_release('', 'Respin', '')
         assert isinstance(got, fedfind.release.RespinRelease)
-        assert got.release == '24'
+        assert got.release == '31'
         assert got.milestone == 'Respin'
-        assert got.compose == '20161120'
+        assert got.compose == '20200427'
         # getting a release from the fake compose ID should get the
         # same release
         got2 = fedfind.release.get_release(cid=got.cid)
         assert got.version == got2.version
         # getting a release with the old form of fake compose ID also
-        got3 = fedfind.release.get_release(cid='FedoraRespin-24-20161120.0')
+        got3 = fedfind.release.get_release(cid='FedoraRespin-31-20200427.0')
         assert got.version == got3.version
         # getting a release from the URL should get the same release
         got4 = fedfind.release.get_release(
             url='https://dl.fedoraproject.org/pub/alt/live-respins/')
         assert got4.version == got3.version
+        # new dist style (should match others)
+        got5 = fedfind.release.get_release('', '', '', dist='FedoraRespin')
+        assert got5.version == got4.version
         # trying to get RespinRelease with a non-matching release
         # or compose should raise an error
         with pytest.raises(ValueError):
@@ -709,35 +932,6 @@ class TestRelease:
         with pytest.raises(ValueError):
             fedfind.release.get_release(cid='FedoraRespin-25-updates-20170305.0')
 
-    @pytest.mark.parametrize(
-        ("release", "milestone", "compose", "respin", "cid"),
-        [
-            # Branched nightly.
-            (23, 'Branched', 'datestr', 1, 'Fedora-23-datestr.n.1'),
-            # Branched modular nightly.
-            (27, 'Branched', 'datestr', 1, 'Fedora-Modular-27-datestr.n.1'),
-            # Rawhide nightly.
-            ('Rawhide', '', 'datestr', 2, 'Fedora-Rawhide-datestr.n.2'),
-            # Rawhide modular nightly.
-            ('Rawhide', '', 'datestr', 2, 'Fedora-Modular-Rawhide-datestr.n.2'),
-            # Atomic nightly.
-            (24, 'Atomic', 'datestr', 0, 'Fedora-Atomic-24-datestr.0'),
-            # Docker nightly.
-            (25, 'Docker', 'datestr', 0, 'Fedora-Docker-25-datestr.0'),
-            # Cloud nightly.
-            (25, 'Cloud', 'datestr', 0, 'Fedora-Cloud-25-datestr.0'),
-        ]
-    )
-    def test_get_release_cid_passthru(self, release, milestone, compose, respin, cid):
-        """Check compose ID is passed through from get_release to the
-        Release instance in various cases.
-        """
-        datestr = datetime.date.today().strftime('%Y%m%d')
-        compose = compose.replace('datestr', datestr)
-        cid = cid.replace('datestr', datestr)
-        got = fedfind.release.get_release(release, milestone, compose, respin, cid=cid)
-        assert got._cid == cid
-
     @mock.patch('fedfind.helpers.urlopen_retries', urlopen_fake_package)
     def test_get_package_nvras_fc1(self):
         """FC1: no /Everything, no /source, no split-by-initials."""
@@ -788,19 +982,106 @@ class TestRelease:
             'fakepackage': ''
         }
 
+    @mock.patch('fedfind.helpers.urlopen_retries', urlopen_fake_package)
+    @mock.patch('fedfind.release.BranchedNightly.cid', "Fedora-24-20160321.n.0")
+    def test_get_package_nvras_f24(self):
+        """F24: Different layout again under /Everything. mock values
+        are same as for F18.
+        """
+        rel = fedfind.release.get_release(24, 'Branched', '20160321', 0)
+        pkgs = rel.get_package_nvras(['amanda', 'anaconda', 'bash', 'fakepackage'])
+        assert pkgs == {
+            'amanda': 'amanda-3.3.2-2.fc18.src',
+            'anaconda': 'anaconda-18.37.11-1.fc18.src',
+            'bash': 'bash-4.2.39-3.fc18.src',
+            'fakepackage': ''
+        }
+
+    @mock.patch('fedfind.helpers.urlopen_retries', urlopen_fake_package)
+    @mock.patch('fedfind.release.BranchedModularNightly.cid', 'Fedora-Modular-27-20171115.n.1')
+    def test_get_package_nvras_mod_f27(self):
+        """Modular F27: No /Everything tree, only /Server. mock values
+        are same as for F18.
+        """
+        rel = fedfind.release.get_release(cid="Fedora-Modular-27-20171115.n.1")
+        pkgs = rel.get_package_nvras(['amanda', 'anaconda', 'bash', 'fakepackage'])
+        assert pkgs == {
+            'amanda': 'amanda-3.3.2-2.fc18.src',
+            'anaconda': 'anaconda-18.37.11-1.fc18.src',
+            'bash': 'bash-4.2.39-3.fc18.src',
+            'fakepackage': ''
+        }
+
+    @mock.patch('fedfind.helpers.pdc_query', return_value=PDCPACKAGES)
+    @mock.patch('fedfind.release.BranchedNightly.cid', "Fedora-24-20160321.n.0")
+    def test_get_package_nevras_pdc_f24(self, fakepdc):
+        """Test get_package_nevras_pdc (like get_package_nvras, but
+        uses PDC and gets epochs) with a Fedora 24 compose.
+        """
+        rel = fedfind.release.get_release(24, 'Branched', '20160321', 0)
+        pkgs = rel.get_package_nevras_pdc(['amanda', 'anaconda', 'bash', 'fakepackage'])
+        assert pkgs == {
+            'amanda': 'amanda-0:3.3.8-1.fc24.src',
+            'anaconda': 'anaconda-0:24.13-1.fc24.src',
+            'bash': 'bash-0:4.3.42-4.fc24.src',
+            'fakepackage': ''
+        }
+        fakepdc.assert_called_with(
+            'rpms',
+            [
+                ('compose', 'Fedora-24-20160321.n.0'),
+                ('arch', 'src'),
+                ('name', '^amanda$'),
+                ('name', '^anaconda$'),
+                ('name', '^bash$'),
+                ('name', '^fakepackage$')
+            ]
+        )
+
+    @mock.patch('fedfind.helpers.pdc_query', return_value=None)
+    def test_all_paths_sizes(self, fakepdc, http):
+        """Check that all_paths_sizes works correctly with both
+        formats. Note the test data has a line edited to test this.
+        We mock out pdc_query so we don't get 'real' metadata from
+        PDC and are forced back on synthesis, and then use
+        all_images to check that all_paths_sizes and metadata both
+        do their job properly.
+        """
+        rel = fedfind.release.get_release(25)
+        imgs = rel.all_images
+        for img in imgs:
+            if 'Fedora-Server-dvd-x86_64-25-1.3.iso' in img['path']:
+                # this is the line that's edited to have a size
+                assert img['size'] == 2018508800
+            else:
+                # old-style line, size should not be in dict at all
+                assert 'size' not in img
+
+    @mock.patch('fedfind.release.CurrentRelease.exists', True)
     def test_all_stable(self, http, pdc):
         """Test that we get the expected image dicts for all stable
         releases, using the test HTTP server and home directory and
         the PDC query mock. Also check there are no missing expected
-        images (as a test for check_expected).
+        images (as a test for check_expected). Note this test is
+        # sensitive to the stupid 'current/archive' detection in
+        # get_release: it will fail if the hardcoded cutoff there
+        # doesn't match the data in allstable.json.
         """
         ref = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'allstable.json')
         with codecs.open(ref, encoding='utf-8') as reffh:
             expected = json.loads(reffh.read())
+        # one thing we can handily check here, each time we bump the
+        # backing data for this test to include a new release, is if
+        # any sneaky new subvariants showed up...
+        subvs = [subv.name for subv in fedfind.const.SUBVARIANTS]
         for relnum in expected:
             rel = fedfind.release.get_release(relnum)
             imgs = sorted(rel.all_images, key=lambda x: x['path'])
             assert expected[relnum] == imgs
+            for img in imgs:
+                subv = img.get('subvariant')
+                if subv:
+                    assert subv in subvs
             # we have to force the release to 'exist' or else
             # check_expected bails
             rel._exists = True
@@ -815,53 +1096,32 @@ class TestRelease:
         """
         rel = fedfind.release.RespinRelease()
         imgs = rel.all_images
-        assert len(imgs) == 7
+        assert len(imgs) == 9
         # now ditch source, as we don't care much about it
         imgs = [img for img in imgs if img['subvariant'] != 'Source']
-        assert len(imgs) == 6
+        assert len(imgs) == 8
         assert all(img['type'] == 'live' for img in imgs)
         # this is a lie, but it's the lie we decided to tell
         assert all(img['variant'] == 'Spins' for img in imgs)
         assert all(img['arch'] == 'x86_64' for img in imgs)
-        exp = ['Cinnamon', 'KDE', 'LXDE', 'Mate', 'Workstation', 'Xfce']
+        exp = ['Cinnamon', 'KDE', 'LXDE', 'LXQt', 'Mate', 'SoaS', 'Workstation', 'Xfce']
         got = sorted([img['subvariant'] for img in imgs])
         assert got == exp
 
-    # FIXME: Pungi 4 PDC-based get_pkg_nvras method is temporarily
-    # disabled due to
-    # https://github.com/fedora-infra/pdc-updater/issues/10 so this
-    # is using the same mirror-scrape-y method for now. Pungi 4
-    # composes have a different layout again under /Everything, we
-    # use the same mock values as F18.
-    #@mock.patch('fedfind.helpers.pdc_query', return_value=PDCPACKAGES)
-    @mock.patch('fedfind.helpers.urlopen_retries', urlopen_fake_package)
-    @mock.patch('fedfind.release.BranchedNightly.cid', "Fedora-24-20160321.n.0")
-#    def test_get_package_nvras_pungi4(self, fakepdc):
-    def test_get_package_nvras_pungi4(self):
-        """Pungi 4 compose: we hit PDC."""
-        rel = fedfind.release.get_release(24, 'Branched', '20160321', 0)
-        pkgs = rel.get_package_nvras(['amanda', 'anaconda', 'bash', 'fakepackage'])
-        assert pkgs == {
-            'amanda': 'amanda-3.3.2-2.fc18.src',
-            'anaconda': 'anaconda-18.37.11-1.fc18.src',
-            'bash': 'bash-4.2.39-3.fc18.src',
-            'fakepackage': ''
-        }
-#        assert pkgs == {
-#            'amanda': 'amanda-0:3.3.8-1.fc24.src',
-#            'anaconda': 'anaconda-0:24.13-1.fc24.src',
-#            'bash': 'bash-0:4.3.42-4.fc24.src',
-#        }
-#        fakepdc.assert_called_with(
-#            'rpms',
-#            [
-#                ('compose', 'Fedora-24-20160321.n.0'),
-#                ('arch', 'src'),
-#                ('name', 'amanda'),
-#                ('name', 'anaconda'),
-#                ('name', 'bash'),
-#                ('name', 'fakepackage')
-#            ]
-#        )
+    def test_compose_urls(self, http):
+        """Check that get_release(url=) works sanely for a list of 180
+        days worth of compose URLs. List is generated by fedfindloc.py
+        script.
+        """
+        urlfn = 'compose-urls-20200428.json'
+        urlfp = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', urlfn)
+        with codecs.open(urlfp, encoding='utf-8') as urlfh:
+            urls = json.loads(urlfh.read())
+        for url in urls:
+            try:
+                assert fedfind.release.get_release(url=url)
+            except fedfind.exceptions.UnsupportedComposeError:
+                pass
+#            except fedfind.exceptions.UrlMatchError as err:
 
 # vim: set textwidth=100 ts=8 et sw=4:
